"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bootstrapBaseWebApp = exports.applyExpressCompatibilityRecommendations = exports.createNestWebApp = exports.setupGlobalFilters = exports.buildFastifyAdapter = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const platform_fastify_1 = require("@nestjs/platform-fastify");
const testing_module_1 = require("@nestjs/testing/testing-module");
const class_validator_1 = require("class-validator");
const nestjs_i18n_1 = require("@saas-buildkit/nestjs-i18n");
const nestjs_pino_1 = require("nestjs-pino");
const typeorm_transactional_1 = require("typeorm-transactional");
const common_2 = require("typeorm-transactional/dist/common");
const crypto_1 = require("@softkit/crypto");
const typeorm_extension_1 = require("typeorm-extension");
const exceptions_1 = require("@softkit/exceptions");
const validation_1 = require("@softkit/validation");
const app_1 = require("./config/app");
const swagger_utils_1 = require("@softkit/swagger-utils");
const typeorm_1 = require("@softkit/typeorm");
const logger_1 = require("@softkit/logger");
const i18n_1 = require("@softkit/i18n");
const server_http_client_1 = require("@softkit/server-http-client");
const helmet_1 = require("@fastify/helmet");
const typeorm_2 = require("typeorm");
const functions_1 = require("./utils/functions");
function buildFastifyAdapter() {
    return new platform_fastify_1.FastifyAdapter({
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        genReqId: (req) => {
            // eslint-disable-next-line security/detect-object-injection
            const requestId = req.headers[server_http_client_1.REQUEST_ID_HEADER];
            return requestId || (0, crypto_1.generateRandomId)();
        },
    });
}
exports.buildFastifyAdapter = buildFastifyAdapter;
function setupGlobalFilters(app, httpAdapterHost) {
    app.useGlobalFilters(new exceptions_1.AnyExceptionFilter(httpAdapterHost), new exceptions_1.OverrideDefaultNotFoundFilter(httpAdapterHost), new exceptions_1.OverrideDefaultForbiddenExceptionFilter(httpAdapterHost), 
    // todo generalize
    new typeorm_1.PostgresDbQueryFailedErrorFilter(httpAdapterHost), new exceptions_1.HttpExceptionFilter(httpAdapterHost), new nestjs_i18n_1.I18nValidationExceptionFilter({
        responseBodyFormatter: i18n_1.responseBodyFormatter,
        detailedErrors: true,
    }));
}
exports.setupGlobalFilters = setupGlobalFilters;
async function createNestWebApp(
// eslint-disable-next-line @typescript-eslint/no-explicit-any
module, 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
originalModule) {
    const isTestingModule = module instanceof testing_module_1.TestingModule;
    if (isTestingModule && originalModule === undefined) {
        throw new Error('If you are using TestingModule, you must pass the original module as the second argument');
    }
    return isTestingModule
        ? module.createNestApplication(buildFastifyAdapter())
        : await core_1.NestFactory.create(module, buildFastifyAdapter(), {});
}
exports.createNestWebApp = createNestWebApp;
function applyExpressCompatibilityRecommendations(fastifyInstance) {
    // this is a recommendation from fastify to improve compatibility with express middlewares
    fastifyInstance
        .addHook('onRequest', async (req) => {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        req.socket['encrypted'] = process.env.NODE_ENV === 'production';
    })
        .decorateReply('setHeader', 
    /* istanbul ignore next */ function (name, value) {
        this.header(name, value);
    })
        .decorateReply('end', 
    /* istanbul ignore next */ function () {
        this.send('');
    });
}
exports.applyExpressCompatibilityRecommendations = applyExpressCompatibilityRecommendations;
function setupGlobalInterceptors(app) {
    app.useGlobalInterceptors(new logger_1.LoggingInterceptor());
    app.useGlobalInterceptors(new common_1.ClassSerializerInterceptor(app.get(core_1.Reflector)));
    app.useGlobalInterceptors(new nestjs_pino_1.LoggerErrorInterceptor());
}
async function runDatabaseSeeders(app, logger, shouldRunSeeds) {
    if (!shouldRunSeeds) {
        return;
    }
    const ds = (0, functions_1.callOrUndefinedIfException)(() => app.get(typeorm_2.DataSource));
    const seeders = app.get(typeorm_1.TYPEORM_SEEDERS_TOKEN);
    const factories = app.get(typeorm_1.TYPEORM_FACTORIES_TOKEN);
    if (seeders.length === 0) {
        return logger.warn('Warning: No seeders found. Ensure you have provided seeders if you are expecting database seeding to occur.');
    }
    if (ds instanceof typeorm_2.DataSource) {
        await (0, typeorm_extension_1.runSeeders)(ds, {
            seeds: seeders,
            factories,
        });
    }
    else {
        logger.warn('Seems like run seeds is enabled, but there is no data source provided, this seems like a mistake. Please review or disable seed run');
    }
}
async function bootstrapBaseWebApp(
// eslint-disable-next-line @typescript-eslint/no-explicit-any
module, 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
originalModule) {
    // todo wait for the pr in this package to be merged
    //  transition to use AsyncCls instead of ClsHook
    const transactionalContext = (0, common_2.getTransactionalContext)();
    // this is needed for tests to prevent multiple initializations
    if (!transactionalContext) {
        (0, typeorm_transactional_1.initializeTransactionalContext)();
    }
    const app = await createNestWebApp(module, originalModule);
    const fastifyInstance = app.getHttpAdapter().getInstance();
    applyExpressCompatibilityRecommendations(fastifyInstance);
    app.register(helmet_1.fastifyHelmet, {});
    (0, class_validator_1.useContainer)(app.select(originalModule || module), {
        fallbackOnErrors: true,
    });
    app.enableShutdownHooks();
    const logger = app.get(nestjs_pino_1.Logger);
    app.useLogger(logger);
    app.flushLogs();
    const httpAdapterHost = app.get(core_1.HttpAdapterHost);
    app.useGlobalPipes(new nestjs_i18n_1.I18nValidationPipe(validation_1.DEFAULT_VALIDATION_OPTIONS));
    // so first global one and then narrow
    setupGlobalFilters(app, httpAdapterHost);
    setupGlobalInterceptors(app);
    const appConfig = app.get(app_1.AppConfig);
    const dbConfig = (0, functions_1.callOrUndefinedIfException)(() => app.get(typeorm_1.DbConfig));
    const swaggerConfig = (0, functions_1.callOrUndefinedIfException)(() => app.get(swagger_utils_1.SwaggerConfig));
    if (appConfig.prefix) {
        app.setGlobalPrefix(appConfig.prefix);
    }
    app.enableVersioning({
        type: common_1.VersioningType.URI,
    });
    if (swaggerConfig instanceof swagger_utils_1.SwaggerConfig) {
        const swaggerSetup = (0, swagger_utils_1.setupSwagger)(swaggerConfig, app, appConfig.prefix);
        const swaggerPath = `${appConfig.prefix}${swaggerConfig.swaggerPath}`;
        if (swaggerSetup) {
            logger.log(`Swagger is listening on ${swaggerPath}`);
        }
        else {
            logger.log(`Swagger is disabled by config, skipping...`);
        }
    }
    else {
        logger.debug(`SwaggerConfig instance is not provided so swagger turned off by default, skipping... Details: %o`, swaggerConfig);
    }
    if (dbConfig instanceof typeorm_1.DbConfig) {
        await runDatabaseSeeders(app, logger, dbConfig.runSeeds);
    }
    await app.listen(appConfig.port, '0.0.0.0');
    logger.log(`App successfully started. Listening on port ${appConfig.port}`);
    return app;
}
exports.bootstrapBaseWebApp = bootstrapBaseWebApp;
//# sourceMappingURL=setup-web-app.js.map